@@ -7,7 +7,7 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants ':all';
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.19.0';
 
 extends 'Lemonldap::NG::Common::PSGI::Request';
 
@@ -44,6 +44,10 @@ has mustRedirect => ( is => 'rw' );
 # authentication)
 has noLoginDisplay => ( is => 'rw' );
 
+# Portal URL, can be dynamic
+
+has portal => ( is => 'rw' );
+
 # Store URL for redirections
 has urldc                  => ( is => 'rw' );
 has postUrl                => ( is => 'rw' );
@@ -74,7 +78,10 @@ has lockTime => ( is => 'rw' );
 
 # Security
 #
-# Captcha
+# Captcha HTML code to display in forms
+has captchaHtml => ( is => 'rw' );
+
+# DEPRECATED: 2.0 captcha compatibility
 has captcha => ( is => 'rw' );
 
 # Token
@@ -132,6 +139,7 @@ sub error_type {
                 PE_REGISTERFORMEMPTY,
                 PE_PP_CHANGE_AFTER_RESET,
                 PE_RESETCERTIFICATE_FORMEMPTY,
+                PE_RETRY_2FA,
             )
         )
       );
@@ -146,7 +154,9 @@ sub init {
     my ( $self, $conf ) = @_;
     $self->{$_} = {} foreach (qw(data customParameters sessionInfo pdata));
     $self->{$_} = [] foreach (qw(respCookies));
-    if ( my $tmp = $self->userData->{ $conf->{whatToTrace} } ) {
+    if ( $conf->{whatToTrace}
+        and my $tmp = $self->userData->{ $conf->{whatToTrace} } )
+    {
         $self->user($tmp);
     }
 }
