@@ -4,10 +4,10 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants 'PE_OK';
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.19.0';
 
 extends qw(
-  Lemonldap::NG::Common::Module
+  Lemonldap::NG::Portal::Main::UserDB
   Lemonldap::NG::Portal::Lib::Remote
 );
 
@@ -29,8 +29,4 @@ sub setSessionInfo {
     return PE_OK;
 }
 
-sub setGroups {
-    return PE_OK;
-}
-
 1;
